@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -28,7 +25,7 @@
 
 #include <IOKit/graphics/IOAccelSurfaceConnect.h>
 
-#define IOACCEL_SURFACE_CONTROL_REV	5
+#define IOACCEL_SURFACE_CONTROL_REV	6
 
 typedef struct IOAccelConnectStruct *IOAccelConnect;
 
@@ -44,7 +41,10 @@ IOReturn IOAccelSetSurfaceScale( IOAccelConnect connect, IOOptionBits options,
 IOReturn IOAccelDestroySurface( IOAccelConnect connect );
 
 /* Change the visible region of the accelerated surface */
-IOReturn IOAccelSetSurfaceShape( IOAccelConnect connect, IOAccelDeviceRegion *region, eIOAccelSurfaceShapeBits options );
+IOReturn IOAccelSetSurfaceFramebufferShapeWithBacking( IOAccelConnect connect, IOAccelDeviceRegion *rgn,
+                                            eIOAccelSurfaceShapeBits options, UInt32 framebufferIndex,
+					    IOVirtualAddress backing, UInt32 rowbytes );
+
 IOReturn IOAccelSetSurfaceFramebufferShape( IOAccelConnect connect, IOAccelDeviceRegion *rgn,
                                             eIOAccelSurfaceShapeBits options, UInt32 framebufferIndex );
 
@@ -52,10 +52,13 @@ IOReturn IOAccelSetSurfaceFramebufferShape( IOAccelConnect connect, IOAccelDevic
 IOReturn IOAccelWaitForSurface( IOAccelConnect connect );
 
 /* Get the back buffer of the surface.  Supplies client virtual address. */
-IOReturn IOAccelLockSurface( IOAccelConnect connect, IOAccelSurfaceInformation * info, UInt32 infoSize );
 
-/* Release the lock on the surface's vram */
-IOReturn IOAccelUnlockSurface( IOAccelConnect connect );
+IOReturn IOAccelWriteLockSurfaceWithOptions( IOAccelConnect connect, IOOptionBits options,
+					     IOAccelSurfaceInformation * info, UInt32 infoSize );
+IOReturn IOAccelWriteUnlockSurfaceWithOptions( IOAccelConnect connect, IOOptionBits options );
+IOReturn IOAccelReadLockSurfaceWithOptions( IOAccelConnect connect, IOOptionBits options,
+					    IOAccelSurfaceInformation * info, UInt32 infoSize );
+IOReturn IOAccelReadUnlockSurfaceWithOptions( IOAccelConnect connect, IOOptionBits options );
 
 IOReturn IOAccelQueryLockSurface( IOAccelConnect connect );
 IOReturn IOAccelWriteLockSurface( IOAccelConnect connect, IOAccelSurfaceInformation * info, UInt32 infoSize );
@@ -64,7 +67,6 @@ IOReturn IOAccelReadLockSurface( IOAccelConnect connect, IOAccelSurfaceInformati
 IOReturn IOAccelReadUnlockSurface( IOAccelConnect connect );
 
 /* Flush surface to visible region */
-IOReturn IOAccelFlushSurface( IOAccelConnect connect, IOOptionBits options );
 IOReturn IOAccelFlushSurfaceOnFramebuffers( IOAccelConnect connect, IOOptionBits options, UInt32 framebufferMask );
 
 
